"""
opta_sd.soccer
"""

from .feeds import (
    Commentary, Match, MatchEvent, MatchFacts, MatchPreview,
    MatchStatistics, PassMatrix, PlayerCareer, Possession,
    Rankings, SeasonalStats, Squads, TeamStandings,
    TournamentCalendar, TournamentSchedule, Venues,
)


__all__ = [
    "Commentary", "Match", "MatchEvent", "MatchFacts", "MatchPreview",
    "MatchStatistics", "PassMatrix", "PlayerCareer", "Possession",
    "Rankings", "SeasonalStats", "Squads", "TeamStandings",
    "TournamentCalendar", "TournamentSchedule", "Venues",
]